package no.rune.mavenplugins.dla;

import java.io.File;
import java.io.FileNotFoundException;
import java.util.Arrays;
import java.util.List;


import org.apache.maven.artifact.Artifact;
import org.apache.maven.plugin.MojoExecutionException;
import org.apache.maven.plugin.MojoFailureException;
import org.apache.maven.project.MavenProject;


import org.aspectj.bridge.IMessage;
import org.aspectj.bridge.MessageHandler;
import org.aspectj.tools.ajc.Main;
import org.codehaus.mojo.aspectj.AjcHelper;


import no.rune.mavenplugins.dla.aspectj.ConsoleReporterMessageHandler;
import no.rune.shellexecute.AjcArgument;
import no.rune.shellexecute.Arguments;
import no.rune.shellexecute.SpaceDelimitedArguments;
import no.rune.mavenplugins.dla.utils.ExtensionFileFilter;


/**
 * This will weave the aspects and report on specific warnings and
 * errors regarding design issues.
 *
 * @author Rune Flobakk
 * @phase test
 * @goal verify
 * @requiresDependencyResolution compile
 *
 * @todo Enable inclusion/exclusion filter for AspectJ sources if being generated
 * to src/main/aspect directory. This is to being able to avoid compiling other aspects
 * that may be present there, if the project uses AspectJ for general AOP.
 *
 */
public class DesignVerifierMojo extends AbstractDlaPluginMojo
{


    /**
     * File filter accepting Java source files (*.java)
     */
    protected static final ExtensionFileFilter JAVA_SOURCES =
    new ExtensionFileFilter()
    {
        /**
         * @see java.io.FileFilter#accept(java.io.File)
         */
        public boolean accept( File pathname )
        {
            return gotExtension( pathname, "java" );
        }
    };



    /**
     * File filter accepting AspectJ sources (*.aj and *.java)
     */
    protected static final ExtensionFileFilter ASPECT_SOURCES =
    new ExtensionFileFilter()
    {
        /**
         * @see java.io.FileFilter#accept(java.io.File)
         */
        public boolean accept( File pathname )
        {
            return gotExtension( pathname, "java" )
                || gotExtension( pathname, "aj" );
        }
    };




    /**
     * Name of the AspectJ compiler report file.
     *
     * @parameter default-value="aspects.xml"
     * @required
     * @readonly
     */
    protected String reportFile;





    /**
     * The plugin's dependencies.
     *
     * @parameter expression="${plugin.artifacts}"
     * @required
     * @readonly
     */
    protected List<Artifact> pluginDependencyArtifacts;



    /**
     * The directory where the compiled classes and aspects are put.
     *
     * @parameter expression="${project.build.directory}/generated-classes/dla"
     * @required
     */
    protected File compileTarget;



    /**
     * The current project.
     *
     * @parameter expression="${project}"
     * @required
     * @readonly
     */
    protected MavenProject project;



    /**
     * If the build should fail on DLA errors.
     *
     * @parameter default-value="true"
     * @required
     */
    protected boolean failOnError;



    /**
     * If the build should fail on DLA warnings.
     *
     * @parameter default-value="false"
     * @required
     */
    protected boolean failOnWarning;



    /**
     * @see org.apache.maven.plugin.Mojo#execute()
     */
    public void execute() throws MojoExecutionException, MojoFailureException
    {
        log = getLog();

        if( ! "java".equals(
            project.getArtifact().getArtifactHandler().getLanguage() ) )
        {
            log.warn( "Uncompatible project! The dla-maven-plugin only " +
                "supports Java projects, and will thus not run. " );
            return;
        }


        log.info( "Testing design adherence" );

        log.debug( "AspectJ Runtime: " + getPluginDependencyFile( "aspectjrt" ) );
        log.debug( "Java sources: " + this.javaSourceDirectory );
        log.debug( "aspectDirectory: " + this.getAspectDirectory() );
        log.debug( "compileTarget: " + this.compileTarget );




        //Create an instance of the AspectJ compiler, assemble the arguments,
        //assign a call-back handler for compiler messages, and finally run it 
        Main ajCompiler = new Main();
        Arguments args = getAjcArguments();
        MessageHandler handler = new ConsoleReporterMessageHandler( log );
        log.debug( "Running: ajc " + args );
        ajCompiler.run( args.getArguments(), handler );

        if( handler.hasAnyMessage( IMessage.WARNING, true ) )
        {
            if( failOnWarning || ( failOnError &&
                    handler.hasAnyMessage( IMessage.ERROR, true ) ) )
                throw new MojoFailureException("There were design violations!");
            else
                log.warn( "There were design violations!" );
        }
        else
            log.debug( "No design violations detected." );

    }






    /**
     * Use this method to generate the required classpath for the
     * AspectJ compiler. It will resolve the aspectjrt library in the local
     * Maven repository, and add direct dependencies for the project.
     *
     * @return the classpath string
     */
    private String getClassPath() throws FileNotFoundException
    {
        File aspectjrt = getPluginDependencyFile( "aspectjrt" );

        if( ! aspectjrt.exists() )
            throw new FileNotFoundException(
                "Could not resolve the aspectj runtime library in local " +
                "repository at " + aspectjrt );

        List outputDirectories = Arrays.asList(
            new String[]{project.getBuild().getOutputDirectory()});
        
        return aspectjrt + File.pathSeparator +
            AjcHelper.createClassPath( this.project, outputDirectories );
    }



    /**
     * This method will assemble and return required arguments
     * for the AspectJ compiler.
     * @return the arguments
     */
    protected Arguments getAjcArguments() throws MojoExecutionException
    {
        Arguments args = new SpaceDelimitedArguments();
        try
        {
            args.addArgument( AjcArgument.OUTCLASSDIR,
                this.compileTarget.getAbsolutePath() );
            args.addArgument( AjcArgument.OUTXMLFILE, reportFile );
            args.addArgument( AjcArgument.SOURCEROOTS,
                javaSourceDirectory + ( (getAspectDirectory().exists())?
                File.pathSeparator + getAspectDirectory() : "") );
            args.addArgument( AjcArgument.CLASSPATH, getClassPath() );

            args.addArgument( AjcArgument.SOURCE, "1.5" );
            args.addArgument( AjcArgument.TARGET, "1.5" );

        }
        catch( Exception e )
        {
            throw new MojoExecutionException( e.getMessage(), e );
        }
        return args;
    }



    /**
     * Get the file in the local repository for a dependency of
     * this plugin.
     *
     * @param artifactId The artifact to look up
     * @return The file located in the local repository, or <code>null</code>
     * if the dependency was not found.
     */
    private File getPluginDependencyFile( String artifactId )
    {
        for( Artifact artifact : pluginDependencyArtifacts )
            if( artifact.getArtifactId().equals( artifactId ) )
                return artifact.getFile();

        return null;
    }

}
