package no.rune.shellexecute;


import java.util.ArrayList;
import java.util.Collection;
import java.util.Map;
import java.util.TreeMap;
import java.util.Map.Entry;


/**
 * Class with common functionality for handling of command line arguments. It
 * is backed up by a Map data structure for holding the arguments.
 * 
 * <p>
 * Classes extending this should override <code>toString</code> to return
 * a correctly formatted string of how the arguments must appear on a
 * command line.
 * </p>
 * 
 * @author Rune Flobakk
 */
public abstract class AbstractArguments implements Arguments
{


    /**
     * A map containing each command argument. In the case of
     * arguments taking following string(s), the post
     * strings are Map values. 
     */
    protected Map<Argument, String> theArguments;

    
    public AbstractArguments()
    {
        theArguments = new TreeMap<Argument, String>();
    }

    
   
  
    
    public void addArgument( Argument arg ) throws MissingPostStringException
    {
        try
        {
            addArgument( arg, null );
        }
        catch( PostStringNotAllowedException e )
        {
            throw new RuntimeException( "Unrecoverable error: " +
                    "PostStringNotAllowedException not supposed to be thrown.");
        }
    }
    

    /**
     * Add (or replace) an argument. 
     * @param arg
     * @param postString
     * @throws PostStringNotAllowedException if trying to add a flag type
     * argument with postString. 
     */
    public void addArgument( Argument arg, String postString )
        throws PostStringNotAllowedException, MissingPostStringException
    {
        if( arg.isFlag() && postString != null )
        {
            throw new PostStringNotAllowedException( "Argument " + arg +
                " is a flag, and cannot contain post string (" + postString + 
                ").");
        }
        else if( ! arg.isFlag() && postString == null )
        {
            throw new MissingPostStringException( "Argument " + arg +
                "requires a post string.");
        }
        theArguments.put( arg, postString );
    }

    
    /**
     * Add more arguments. Existing arguments, if any, will be overwritten.
     * @see no.rune.shellexecute.Arguments#addArguments(Map)
     */
    public void addArguments( Map<Argument, String> arguments )
        throws PostStringNotAllowedException, MissingPostStringException
    {
        for( Entry<Argument, String> entry : arguments.entrySet() )
            addArgument( entry.getKey(), entry.getValue() );
    }
    
    
        

    /**
     * Add more arguments. Existing arguments, if any, will be overwritten.
     * @see no.rune.shellexecute.Arguments#addArguments(no.rune.shellexecute.Argument[])
     */
    public void addArguments( Argument[] args )
        throws MissingPostStringException
    {
        if( args == null || args.length == 0 )
            return;
        for( Argument arg : args )
            addArgument( arg );
    }



    /**
     * Get the post-string for an argument
     * 
     * @param argument
     * @return the post string, <code>null</code> if the argument does not exist
     * @throws PostStringNotAllowedException if the post string for
     * the requested argument
     */
    public String getPostString( Argument argument )
        throws PostStringNotAllowedException
    {
        if( argument.isFlag() )
            throw new PostStringNotAllowedException();
        else if( theArguments.containsKey( argument ) )
            return theArguments.get( argument );
        else
            return null;
    }
    
    
    
    /**
     * @see no.rune.shellexecute.Arguments#getArguments()
     */
    public String[] getArguments()
    {
        Collection<String> args = new ArrayList<String>( theArguments.size() * 2 );
        
        Argument arg;
        for( Map.Entry<Argument, String> entry : theArguments.entrySet() )
        {
            arg = entry.getKey();
            args.add( arg.asString() );
            if( ! arg.isFlag() )
            {
                args.add( entry.getValue() );
            }
        }
        
        return args.toArray( new String[0] );
    }
    
    
    
    /**
     * Get the actual string of arguments as it appears when used in the
     * shell or command line. Classes extending this class should implement
     * any particular formatting requirements of the resulting string, if any.
     * 
     * @see java.lang.Object#toString()
     */
    public abstract String toString();

    
}
