package no.rune.shellexecute;


import java.lang.reflect.Field;



/**
 * The available arguments for the AspectJ compiler (ajc) except for the ones
 * marked as experimental and/or deprecated.
 *  
 * 
 * <h3>Documentation</h3>
 * 
 * <p>
 * The documentation of this <code>enum</code> is slightly adapted from the
 * reference at 
 * <a href="http://www.eclipse.org/aspectj/doc/next/devguide/ajc-ref.html">
 *   http://www.eclipse.org/aspectj/doc/next/devguide/ajc-ref.html
 * </a>, which is provided under the terms and conditions of the 
 * <a href="http://www.eclipse.org/legal/epl/notice.php">
 *     Eclipse Foundation Software User Agreement
 * </a>.
 * </p>
 * 
 * <p>
 * The javadoc tool should be configured to document
 * <code>&#64;customizable</code> tags which describe if an argument also
 * expects following argument(s) for customization.
 * </p>
 *  
 * @author Rune Flobakk
 * 
 * TODO maybe consider if command line arguments is better solved if
 * generalized using a regular hierarchy of
 * classes/interfaces instantiated using a configfile at runtime.
 * Nevertheless, using an <code>enum</code> provides a quick'n'dirty
 * approach to handling command line arguments.
 */
public enum AjcArgument implements Argument
{

    
    /**
     * Accept as source bytecode any .class files in the .jar files or
     * directories on Path. The output will include these classes, possibly as
     * woven with any applicable aspects. Path is a single argument containing
     * a list of paths to zip files or directories, delimited by the
     * platform-specific path delimiter.
     * @customizable paths
     */
    @Customizable
    INPATH( "inpath" ),
    
    

    /**
     * Weave binary aspects from jar files and directories on path into all
     * sources. The aspects should have been output by the same version of the
     * compiler. When running the output classes, the run classpath should
     * contain all aspectpath entries. Path, like classpath, is a single
     * argument containing a list of paths to jar files, delimited by the
     * platform-specific classpath delimiter.
     * @customizable paths
     */
    @Customizable
    ASPECTPATH( "aspectpath" ),

    
    
    /**
     * The file contains a line-delimited list of arguments. Each line in the
     * file should contain one option, filename, or argument string (e.g., a
     * classpath or inpath). Arguments read from the file are inserted into the
     * argument list for the command. Relative paths in the file are calculated
     * from the directory containing the file (not the current working
     * directory). Comments, as in Java, start with <code>//</code> and extend
     * to the end of the line. Options specified in argument files may override
     * rather than
     * extending existing option values, so avoid specifying options like
     * -classpath in argument files unlike the argument file is the only build
     * specification.
     * @customizable file
     */
    @Customizable
    ARGFILE( "argfile" ),
    

    
    /**
     * Put output classes in zip file
     * @customizable output file name.
     */
    @Customizable
    OUTJAR( "outjar"),

    

    /**
     * Generate <code>aop.xml</code> file for load-time weaving with default name.
     */
    OUTXML( "outxml" ),

    

    /**
     * Generate aop.xml file for load-time weaving with custom name.
     * @customizable output file name
     */
    @Customizable
    OUTXMLFILE( "outxmlfile" ),
    

    
    /**
     * Run the compiler continuously. After the initial compilation, the
     * compiler will wait to recompile until it reads a newline from the
     * standard input, and will quit when it reads a 'q'. It will only
     * recompile necessary components, so a recompile should be much faster
     * than doing a second compile. This requires
     * {@link AjcArgument#SOURCEROOTS SOURCEROOTS}.
     */
    INCREMENTAL( "incremental" ),

   
    
    /**
     * Find and build all .java or .aj source files under any directory listed
     * in DirPaths. DirPaths, like classpath, is a single argument containing a
     * list of paths to directories, delimited by the platform- specific
     * classpath delimiter. Required by
     * {@link no.rune.shellexecute.AjcArgument#INCREMENTAL INCREMENTAL}.
     * @customizable paths
     */
    @Customizable
    SOURCEROOTS( "sourceroots" ),

    
    
    /**
     * Generate a build .ajsym file into the output directory. Used for viewing
     * crosscutting references by tools like the AspectJ Browser.
     */
    CROSSREFS( "crossrefs" ),
    
    

    
    /**
     * Set default level for messages about potential programming mistakes in
     * crosscutting code. The level may be ignore, warning, or error. This
     * overrides entries in org/aspectj/weaver/XlintDefault.properties from
     * aspectjtools.jar, but does not override levels set using
     * {@link no.rune.shellexecute.AjcArgument#XLINTFILE}.
     * @customizable level: <code>ignore</code>, </code>warning</code>, or <code>error</code>
     */
    @Customizable
    XLINTLEVEL( "Xlint:" ),

    
    
    /**
     * Specify properties file to set levels for specific crosscutting messages.
     * The property file takes the same property names and values as
     * <code>org/aspectj/weaver/XlintDefault.properties</code> from
     * aspectjtools.jar, which it also overrides.
     * @customizable path to a Java .properties file
     */
    @Customizable
    XLINTFILE( "Xlintfile" ),

    

    /**
     * Emit information on compiler options and usage.
     */
    HELP( "help" ),

    
    
    /**
     * Emit the version of the AspectJ compiler.
     */
    VERSION( "version" ),

    
    
    
    /**
     * Specify where to find user class files. Path is a single argument
     * containing a list of paths to zip files or directories, delimited by
     * the platform-specific path delimiter.
     * @customizable paths
     */
    @Customizable
    CLASSPATH( "classpath" ),

    

    /**
     * Override location of VM's bootclasspath for purposes of evaluating types
     * when compiling. Path is a single argument containing a list of paths to
     * zip files or directories, delimited by the platform-specific path
     * delimiter.
     * @customizable paths
     */
    @Customizable
    BOOTCLASSPATH( "bootclasspath" ),
    

    /**
     * Override location of VM's extension directories for purposes of
     * evaluating types when compiling. Path is a single argument containing
     * a list of paths to directories, delimited by the platform-specific
     * path delimiter.
     * @customizable paths
     */
    @Customizable
    EXTDIRS( "extdirs" ),
    

    /**
     * Specify where to place generated .class files. If not specified, 
     * defaults to the current working dir.
     * @customizable directory
     */
    @Customizable
    OUTCLASSDIR( "d" ),

    
    /**
     * @see no.rune.shellexecute.AjcArgument#OUTCLASSDIR
     */
    @Customizable
    D( "d" ),
    

    
    /**
     * Specify classfile target setting (1.1 to 1.5, default is 1.2)
     * @customizable target version
     */
    @Customizable
    TARGET( "target" ),
    

    
    /**
     * Toggle assertions (1.3, 1.4, or 1.5 - default is 1.4). When using 
     * <code>SOURCE</code> 1.3, an assert() statement valid under Java 1.4 will
     * result in a compiler error. When using <code>SOURCE</code> 1.4, treat
     * assert as a keyword and implement assertions according to the 1.4
     * language spec. When using <code>SOURCE</code> 1.5, Java 5 language
     * features are permitted.
     * @customizable source version
     */
    @Customizable
    SOURCE( "source" ),

 
    /**
     * Emit no warnings (equivalent to 
     * {@link no.rune.shellexecute.AjcArgument#WARN WARN}=<code>none</code>) This does not
     * suppress messages generated by <code>declare warning</code> or 
     * {@link no.rune.shellexecute.AjcArgument#XLINT}.
     */
    NOWARN( "nowarn" ),
    

    /**
     * Emit warnings for any instances of a comma-delimited list of
     * questionable code
     * 
     * @customizable list of questionable code<pre>
     * constructorName        method with constructor name
     * packageDefaultMethod   attempt to override package-default method
     * deprecation            usage of deprecated type or member
     * maskedCatchBlocks      hidden catch block
     * unusedLocals           local variable never read
     * unusedArguments        method argument never read
     * unusedImports          import statement not used by code in file
     * none                   suppress all compiler warnings</pre>
     */
    @Customizable
    WARN( "warn" ),
    

    
    /**
     * Same as {@link no.rune.shellexecute.AjcArgument#WARN WARN}=<code>deprecation</code>
     */
    DEPRECATION( "deprecation" ),
    

    /**
     * Emit no errors for unresolved imports.
     */
    NOIMPORTERROR( "noImportError" ),
    
    
    /**
     * Keep compiling after error, dumping class files with problem methods.
     */    
    PROCEEDONERROR( "proceedOnError" ),
    
    
    /**
     * Debug attributes level indicated by a comma-seperated list of elements,
     * defaults to 'lines,vars,source'.
     * @customizable level of debug <pre>
     * none     no debug info (must be used alone)
     * lines    debug info for lines
     * vars     debug info for variables
     * source   debug info for source
     * </pre>
     */
    @Customizable
    DEBUGATTRIBUTESLEVEL( "g" ),
    
    
    /**
     * @see no.rune.shellexecute.AjcArgument#DEBUGATTRIBUTESLEVEL
     */
    @Customizable
    G( "g" ),
    
    
    /**
     * Preserve all local variables during code generation (to facilitate
     * debugging).
     */
    PRESERVE_ALL_LOCALS( "preserveAllLocals" ),
    
    
    /**
     * Compute reference information.
     */
    REFERENCEINFO( "referenceInfo" ),
    
    
    /**
     * Specify default source encoding format.
     * @customizable format
     */
    @Customizable
    ENCODING( "encoding" ),

    
    /**
     * Emit messages about accessed/processed compilation units.
     */
    VERBOSE( "verbose" ),
    
    
    /**
     * Emit messages about weaving.
     */
    SHOWWEAVEINFO( "showWeaveInfo" ),
    

    /**
     * Specify a log file for compiler messages.
     * @customizable output file name
     */
    @Customizable
    LOG( "log" ),
    
    
    /**
     * Show progress (requires {@link AjcArgument#LOG}).
     */
    PROGRESS( "progress" ),
    
    
    /**
     * Display speed information.
     */
    TIME( "time" ),

    
    /**
     * Do not call System.exit(n) at end of compilation (n=0 if no error).
     */
    NOEXIT( "noExit" ),
    

    /**
     * Repeat compilation process N times (typically to do performance analysis).
     * @customizable compilation repetitions amount 
     */
    @Customizable
    REPEAT( "repeat" ),
    

    /**
     * Causes compiler to terminate before weaving.
     */
    XTERMINATEAFTERCOMPILATION( "XterminateAfterCompilation" ),
    

    /**
     * Causes the compiler to calculate and add the <code>SerialVersionUID</code>
     * field to any type implementing Serializable that is affected by an
     * aspect. The field is calculated based on the class before weaving has
     * taken place.
     */
    XADDSERIALVERSION_UID( "XaddSerialVersionUID" );
    
    
    

    /**
     * The string the ajc compiler expects for this argument.
     */
    private String argString;




    /**
     * @param argString argument string
     */
    private AjcArgument( String argString )
    {
        this.argString = argString;
    }

    
    
    /**
     * @see no.rune.shellexecute.Argument#asString()
     */
    public String asString()
    {
        return "-" + argString;
    }
    

    /**
     * @see no.rune.shellexecute.Argument#isFlag()
     */
    public boolean isFlag()
    {
        try
        {
            Field field = this.getClass().getField( toString() );
            if( field.isAnnotationPresent( Customizable.class ) )
                return false;
            else
                return true;
        }
        catch( NoSuchFieldException e )
        {
            throw new RuntimeException( 
                "Unrecoverable error! AjcArgument: " + e.getMessage() );
        }
    }
    
    
}
