package no.rune.shellexecute;

/**
 * An Argument is a known string used to customize the invocation
 * of an executable, usually from the shell.
 * 
 * <p>The reason for
 * wrapping an argument as a type is the ability to determine
 * if this argument takes following arguments, or is a flag
 * type argument (see {@link Argument#isFlag()}).
 * </p>
 * 
 * <h3>Use with enums</h3>
 * <p>
 * Since command line arguments are known string constants it is suitable
 * to implement as an
 * {@link http://java.sun.com/j2se/1.5.0/docs/guide/language/enums.html enum}
 * type in your program. Thus, when writing
 * the <code>enum</code> use:<br/> &nbsp;&nbsp;
 * <code>public enum ExecutableArgument implements Argument</code><br/>
 * (substitute ExecutableArgument with your prefered <code>enum</code> name),
 * and list the constants as names of each possible argument with its string
 * as parameter. E.g: <br/> &nbsp;&nbsp;
 * <code>HELP( "--help" ), VERBOSE( "-v" );</code><br/>
 * In addition, the <code>enum</code> must implement the methods if this
 * interface.
 * </p>
 * <p>
 * Note: if the executable does not used "named" arguments, e.g. it solely
 * depends on positioning of its arguments, this framework is <em>not</em>
 * suitable for argument handling. 
 * </p>
 * 
 * 
 * 
 * @author Rune Flobakk
 */
public interface Argument
{
    /**
     * Determine if this argument is a flag type argument, i.e. an
     * argument which does not expect any further arguments.
     * @return true or false if this is a flag argument or not respectively
     */
    public boolean isFlag();
    
    
    /**
     * Gets the string representation of this argument as it must
     * appear on the command line.
     * 
     * @return the String 
     */
    public String asString();
}
