package no.rune.mavenplugins.xslt;

import java.io.File;
import java.io.FileFilter;
import java.util.Collection;
import java.util.Collections;
import java.util.LinkedList;


import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.TransformerFactoryConfigurationError;
import javax.xml.transform.stream.StreamSource;


import org.apache.maven.plugin.AbstractMojo;
import org.apache.maven.plugin.MojoExecutionException;
import org.apache.maven.plugin.MojoFailureException;
import org.apache.maven.plugin.logging.Log;


/**
 * This MOJO validates XSLT stylesheets.
 * 
 * @author Rune Flobakk
 * @goal validate
 * @phase validate
 */
public class XsltValidatorMojo extends AbstractMojo
{

    /**
     * The default extensions of XSLT files to search for
     */
    protected static final String[] DEFAULT_EXTENSIONS = {"xsl", "xslt"};

    
    
    /**
     * Custom extensions of XSLT files.
     * 
     * @parameter
     */
    protected String[] extensions;

    
    /**
     * The source directory to look for XSLT stylesheets. The plugin will search
     * for .xsl and .xslt files if none are given explicitly using the
     * stylesheets configuration parameter.
     * 
     * @parameter expression="${basedir}/src/main/resources/xslt"
     * @required 
     */
    protected File directory;
    

    /**
     * The stylesheets to validate.
     * @parameter
     */
    protected String[] stylesheets;
    
    
    /**
     * @see org.apache.maven.plugin.Mojo#execute()
     */
    public void execute() throws MojoExecutionException, MojoFailureException
    {
        Log log = getLog();
        log.info( "Validateing XSLT stylesheets." );
        
        
        if( extensions == null )
            extensions = DEFAULT_EXTENSIONS;
        
        

        //populate every xsl and xslt file in directory if not given explicitly
        if( stylesheets == null )
        {
            FileFilter filter = new ExtensionFileFilter()
            {
                public boolean accept( File file )
                {
                    for( String ext : extensions )
                        if( gotExtension( file, ext ) )
                            return true;
                    
                    return false;
                }
            };
            stylesheets = recursiveScanDir( directory, filter );
        }
             
        log.debug( "Found " + stylesheets.length + " XSLT stylesheets.");
        

        File xsltFile = null;
        try
        {
            StreamSource xsltSource;
            for( String stylesheet : stylesheets )
            {
                xsltFile = new File( directory, stylesheet );
                xsltSource = new StreamSource( xsltFile );
                log.debug( "Validating " + xsltFile.getName() );
                TransformerFactory.newInstance().newTransformer( xsltSource );
            }
        }
        catch( TransformerConfigurationException e )
        {
            throw new MojoFailureException( "Error validating " +
                xsltFile.getName() + ": " + e.getMessage() );
        }
        catch( TransformerFactoryConfigurationError e )
        {
            throw new MojoFailureException( "Error validating " +
                xsltFile.getName() + ": " + e.getMessage() );
        }
        
        
    }
    
    
    
    /**
     * Retrive a list of all files in a directory and its subdirectories.
     * @param directory The directory to scan.
     * @param filter any FileFilter to use.
     * @return array containing Strings of the file names.
     */
    private static String[] recursiveScanDir( File directory, FileFilter filter )
    {
        Collection<String> files = new LinkedList<String>();
        
        for( File file : directory.listFiles( filter ) )
        {
            if( file.isDirectory() )
                Collections.addAll( files, recursiveScanDir( file, filter ) );
            else
                files.add( file.getName() );
        }
        
        
        return files.toArray( new String[]{} );
    }

}
